package com.finecosoft.aop.pointcutprogrammed.dynamicpc;

import org.aopalliance.aop.Advice;
import org.springframework.aop.Advisor;
import org.springframework.aop.Pointcut;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.aop.support.DefaultPointcutAdvisor;

import com.finecosoft.aop.pointcutprogrammed.LogInterceptor;
import com.finecosoft.aop.pointcutprogrammed.NotToBeDecoratedClass;
import com.finecosoft.aop.pointcutprogrammed.ToBeDecoratedClass;

public class DynamicPointcutExample {

	public static void main(String[] args) {
		ToBeDecoratedClass toBeDecoratedClass = new ToBeDecoratedClass();
		NotToBeDecoratedClass notToBeDecoratedClass = new NotToBeDecoratedClass();
		
		ToBeDecoratedClass proxyDecoratedClass;
		NotToBeDecoratedClass proxyNotToBeDecoratedClass;
		
		Pointcut pointcut = new DynamicPointcut();
		Advice advice = new LogInterceptor();
		Advisor advisor = new DefaultPointcutAdvisor(pointcut, advice);
		
		ProxyFactory proxyFactory = new ProxyFactory();
		proxyFactory.addAdvisor(advisor);
		proxyFactory.setTarget(toBeDecoratedClass);
		
		proxyDecoratedClass = (ToBeDecoratedClass) proxyFactory.getProxy();
		
		proxyFactory = new ProxyFactory();
		proxyFactory.addAdvisor(advisor);
		proxyFactory.setTarget(notToBeDecoratedClass);
		
		proxyNotToBeDecoratedClass = (NotToBeDecoratedClass) proxyFactory.getProxy();

		System.out.println("ToBeDecoratedClass, printInteger(5)");
		proxyDecoratedClass.printInteger(5);
		
		System.out.println("ToBeDecoratedClass, printInteger(25)");
		proxyDecoratedClass.printInteger(25);
		
		System.out.println("ToBeDecoratedClass, rundomSleep");
		proxyDecoratedClass.rundomSleep();
		
		System.out.println("NotToBeDecoratedClass, printInteger(5)");
		proxyNotToBeDecoratedClass.printInteger(5);
		
		System.out.println("NotToBeDecoratedClass, printInteger(25)");
		proxyNotToBeDecoratedClass.printInteger(25);
		
		System.out.println("NotToBeDecoratedClass, rundomSleep");
		proxyNotToBeDecoratedClass.rundomSleep();

	}

}
